@@ -3,7 +3,6 @@ package Tapper::Installer::Precondition::Image;
 use strict;
 use warnings;
 
-use Method::Signatures;
 use Moose;
 extends 'Tapper::Installer::Precondition';
 
@@ -25,6 +24,158 @@ Tapper::Installer::Precondition::Package - Install a package to a given location
 
 =cut
 
+=head2 get_partition_number
+
+Get the partition part of grub notation of a given device file eg. /dev/hda1.
+
+@param string - partition number
+
+@return int - grub device notation
+
+=cut
+
+sub get_partition_number
+{
+        my ($self, $device_file) = @_;
+	my ($partition_number) = $device_file =~ m/(\d+)/;
+	$partition_number--;
+	return $partition_number;
+}
+
+=head2 get_grub_device
+
+Get the disc part of grub notation of a given device file eg. /dev/hda1.
+
+@param string - device file name
+
+@return success - (0, grub device notation)
+@return eror    - (1, error string)
+
+=cut
+
+sub get_grub_device
+{
+        my ($self, $device_file) = @_;
+        my $basedir = $self->cfg->{paths}{base_dir};
+        my $error = $self->log_and_exec("/usr/sbin/grub-install","--recheck",
+                                        "--root-directory=$basedir","--no-floppy","$device_file");
+        my ($grub_device) = $device_file =~ m/[hs]d([a-z])/;
+        $grub_device      =~ tr/[a-j]/[0-9]/;
+	if ($grub_device eq "") {
+                $self->log->warn( "Grub device not found, took '0'");
+                $grub_device = 0;
+  	}
+	return (0, $grub_device);
+}
+
+
+
+=head2 configure_fstab
+
+Write fstab on installed system based upon the installed images and
+partitions.
+
+@return success - 0
+@return error   - error string
+
+=cut
+
+sub configure_fstab
+{
+        my ($self) = @_;
+	# Creates fstab-entry for the final partition
+
+        $self->log->debug("Configuring fstab to contain installed images");
+        open (my $FSTAB, ">", $self->cfg->{paths}{base_dir}."/etc/fstab") or return "Can't open fstab for appending: $!";
+
+        # write defaults for fstab
+        print $FSTAB "proc\t/proc\tproc\tdefaults\t0 0\n","sysfs\t/sys\tsysfs\tnoauto\t0 0\n";
+
+        foreach my $image (@{$self->images}) {
+                print $FSTAB $image->{partition},"\t",$image->{mount},"\text3\tdefaults\t1 1\n";
+        }
+
+        close $FSTAB or return "Can't write fstab: $!"; # well, cases when close fails are rare but still exist
+        return 0;
+}
+
+
+=head2 generate_user_grub_conf
+
+Generate grub config file menu.lst based upon user provided precondition.
+
+@param string - name of the root partition
+
+@return success - 0
+@return error   - error string
+
+=cut
+
+sub generate_user_grub_conf
+{
+        my ($self, $device_file) = @_;
+        my $mount_point=$self->cfg->{paths}{base_dir};
+        my $conf_string=$self->cfg->{grub};
+
+	my $partition_number = $self->get_partition_number($device_file);
+	my ($error, $grub_device) = $self->get_grub_device( $device_file);
+        return $grub_device if $error;
+
+        $conf_string =~ s/\$root/$device_file/g;
+        $conf_string =~ s/\$grubroot/(hd$grub_device,$partition_number)/g;
+
+        return $self->write_menu_lst($conf_string, "truncate");
+}
+
+
+=head2 generate_grub_menu_lst
+
+Create a grub config file (menu.lst) based on the options in the configuration
+hash. The function is mainly a wrapper for create_menu_lst_entry and
+create_new_menu_lst.
+
+@return success - 0
+@return error   - error string
+
+=cut
+
+sub generate_grub_menu_lst
+{
+        my ($self) = @_;
+        my $retval;
+
+        my $partition = $self->images->[0]->{partition};
+
+        if ($self->cfg->{grub}) {
+                return $retval if $retval = $self->generate_user_grub_conf($partition);
+        } else {
+                return $retval if $retval = $self->create_new_menu_lst();
+                return $retval if $retval = $self->create_menu_lst_entry($partition);
+        }
+	return 0;
+}
+
+
+=head2 prepare_boot
+
+Make installed system ready for boot from hard disk.
+
+@return success - 0
+@return error   - error string
+
+=cut
+
+sub prepare_boot
+{
+        my ($self) = @_;
+        my $retval = 0;
+        return $retval if $retval = $self->configure_fstab();
+        return $retval if $retval = $self->generate_grub_menu_lst( );
+	return $retval if $retval = $self->generate_pxe_grub();
+# 	return $retval if $retval = $self->copy_menu_lst();
+        return 0;
+}
+
 
 =head2 get_device
 
@@ -33,7 +184,7 @@ or $device name (with or without preceding /dev/).
 Doesn't work with dev-mapper.
 
 @param string - device or reference to array with device ids
-@param string - base dir prepended to all paths (testing purpose) 
+@param string - base dir prepended to all paths (testing purpose)
 
 @returnlist success - ( 0, device name string)
 @returnlist error   - ( 1, error string)
@@ -90,41 +241,13 @@ Get the label of a partition to be able to set it again at mkfs.
 
 =cut
 
-method get_partition_label($device_file)
+sub get_partition_label
 {
+        my ($self, $device_file) = @_;
         return $self->log_and_exec("e2label $device_file");
-};
+}
 
 
-=head2 configure_fstab
-
-Write fstab on installed system based upon the installed images and
-partitions.
-
-@return success - 0
-@return error   - error string
-
-=cut
-
-method configure_fstab()
-{
-	# Creates fstab-entry for the final partition
-
-        print STDERR "Tapper::Installer::Precondition::Image.configure_fstab()\n";
-
-        # ss5 sschwigo 2008-09-30 ">" to ">>", real append (does this fix the "missing /var" warnings?)
-        open (my $FSTAB, ">>", $self->cfg->{paths}{base_dir}."/etc/fstab") or return "Can't open fstab for appending: $!";
-
-        # write defaults for fstab
-        print $FSTAB "proc\t/proc\tproc\tdefaults\t0 0\n","sysfs\t/sys\tsysfs\tnoauto\t0 0\n";
-
-        foreach my $image (@{$self->cfg->{images}}) {
-                print $FSTAB $image->{partition},"\t",$image->{mount},"\text3\tdefaults\t1 1\n";
-        }
-
-        close $FSTAB or return "Can't write fstab: $!"; # well, cases when close fails are rare but still exist
-        return 0;
-};
 
 =head2 generate_pxe_grub
 
@@ -145,7 +268,7 @@ sub generate_pxe_grub
 	my ($error, $grub_device) = $self->get_grub_device( $partition );
         return $grub_device if $error;
 
-        
+
 
         my $filename = $self->cfg->{paths}{grubpath}."/$hostname.lst";
         open my $fh, ">", $filename or return "Can not open PXE grub file $filename: $!";
@@ -157,7 +280,7 @@ sub generate_pxe_grub
                   "chainloader (hd$grub_device,$partition_number)+1";
         close $fh or return "Closing PXE grub file $filename of NFS failed: $!";
         return 0;
-};
+}
 
 
 
@@ -173,90 +296,17 @@ using PXE grub.
 
 =cut
 
-method copy_menu_lst()
+sub copy_menu_lst
 {
+        my ($self) = @_;
         my $hostname      = $self->gethostname();
         my $menu_lst_file = $self->cfg->{paths}{base_dir}."/boot/grub/menu.lst";
 
         my $tapper_conf  = $self->cfg->{paths}{grubpath};
 
         return $self->log_and_exec("cp $menu_lst_file $tapper_conf/$hostname.lst");
-};
+}
 
-=head2 get_grub_device
-
-Get the disc part of grub notation of a given device file eg. /dev/hda1.
-
-@param string - device file name
-
-@return success - (0, grub device notation)
-@return eror    - (1, error string)
-
-=cut
-
-method get_grub_device($device_file)
-{
-        (my $grub_search_string = $device_file) =~ s/[0-9]//g;
-        my $mount_point=$self->cfg->{paths}{base_dir};               # root partition will always be mounted there
-        my ($error,$grub_device) = $self->log_and_exec("/usr/sbin/grub-install","--recheck",
-                                                       "--root-directory=$mount_point --no-floppy $device_file | grep $grub_search_string");
-        return ($error,$grub_device) if $error;
-        $grub_device =~ s/[a-z()\/ \s]//g;
-	if ($grub_device eq "") {
-                $self->log->warn( "Grub device not found, took '0'");
-                $grub_device = 0;
-  	}
-	return (0,$grub_device);
-};
-
-
-=head2 get_partition_number
-
-Get the partition part of grub notation of a given device file eg. /dev/hda1.
-
-@param string - partition number
-
-@return int - grub device notation
-
-=cut
-
-method get_partition_number($device_file)
-{
-	(my $partition_number = $device_file) =~ s/[a-z\/]//g;
-	$partition_number--;
-	return $partition_number;
-};
-
-=head2 generate_grub_menu_lst
-
-Create a grub config file (menu.lst) based on the options in the configuration
-hash. The function is mainly a wrapper for create_menu_lst_entry and
-create_new_menu_lst.
-
-@return success - 0
-@return error   - error string
-
-=cut
-
-method generate_grub_menu_lst()
-{
-        my $retval;
-
-        return("First precondition is not the root image")
-          if not $self->cfg->{preconditions}->[0]->{precondition_type} eq 'image'
-            and $self->cfg->{preconditions}->[0]->{mount} eq '/';
-
-        # XXX: use $self->images->[0] and check whether this is correct
-        my $partition = $self->cfg->{preconditions}->[0]->{partition};
-
-        if ($self->cfg->{grub}) {
-                return $retval if $retval = $self->generate_user_grub_conf($partition);
-        } else {
-                return $retval if $retval = $self->create_new_menu_lst();
-                return $retval if $retval = $self->create_menu_lst_entry($partition);
-        }
-	return 0;
-};
 
 =head2 create_menu_lst_entry
 
@@ -270,8 +320,9 @@ named /boot/vmlinuz.
 
 =cut
 
-method create_menu_lst_entry($device_file)
+sub create_menu_lst_entry
 {
+        my ($self, $device_file) = @_;
 	my $partition_number     = $self->get_partition_number($device_file);
 	my ($error,$grub_device) = $self->get_grub_device( $device_file);
         return $grub_device if $error;
@@ -285,7 +336,7 @@ method create_menu_lst_entry($device_file)
 
 
         return $self->write_menu_lst($entry,0);
-};
+}
 
 
 =head2 create_new_menu_lst
@@ -298,14 +349,15 @@ are silently overwritten.
 
 =cut
 
-method create_new_menu_lst()
+sub create_new_menu_lst
 {
+        my ($self) = @_;
         my $base = "serial --unit=0 --speed=115200\n".
           "terminal serial\n".
             "timeout 3\n".
               "default 0\n\n\n";
         return $self->write_menu_lst($base,"truncate");
-};
+}
 
 
 =head2 write_menu_lst
@@ -321,8 +373,9 @@ and testability.
 
 =cut
 
-method write_menu_lst($content, $truncate)
+sub write_menu_lst
 {
+        my ($self, $content, $truncate) = @_;
 	my $menu_lst_file = $self->cfg->{paths}{base_dir}."/boot/grub/menu.lst";
         my $mode = '>>';
         if ($truncate) {
@@ -349,8 +402,9 @@ installer functions so the caller doesn't need to care for preparations.
 
 =cut
 
-method install($image)
+sub install
 {
+        my ($self, $image) = @_;
         my $retval;
 
         # set partition name to the normalized value /dev/*
@@ -364,99 +418,29 @@ method install($image)
         $image->{partition}=$partition;
         $self->log->debug("partition = $partition");
 
-        # moint points are set relative to test system root but installation
-        # needs it relative to current root
-        my $mount_point=$image->{mount};
-        $mount_point = $self->cfg->{paths}{base_dir}.$mount_point;
-	$self->create_mount_point($mount_point);
+        # mount points in image precondition are relative to test system root
+        # installation needs it relative to current root
+        my $mount_point = $self->cfg->{paths}{base_dir}.$image->{mount};
+	$error = $self->makedir($mount_point);
+        return $error if $error;
 
         if ($image->{image}) {
-                return $retval if $retval=$self->copy_image( $partition, $image->{image}, $mount_point);
+                $retval = $self->copy_image( $partition, $image->{image}, $mount_point);
+                return $retval if $retval;
         } else {
                 $self->log->debug("No image to install on $partition, mounting old image to $mount_point");
-                if ($retval=$self->log_and_exec("mount $partition $mount_point")) {
-                        $self->images([ @{$self->images}, $mount_point ]);
-                        return $retval;
-                }
+                $retval = $self->log_and_exec("mount","$partition","$mount_point");
+                return $retval if $retval;
         }
-        push (@{$self->cfg->{images}},$image);
+
+        $self->images([ @{$self->images}, $image ]);
 
         $self->log->debug("Image copied successfully");
 
         return 0;
-};
+}
 
 
-=head2 generate_user_grub_conf
-
-Generate grub config file menu.lst based upon user provided precondition.
-
-@param string - name of the root partition
-
-@return success - 0
-@return error   - error string
-
-=cut
-
-method generate_user_grub_conf($device_file)
-{
-        my $mount_point=$self->cfg->{paths}{base_dir};
-        my $conf_string=$self->cfg->{grub};
-
-	my $partition_number = $self->get_partition_number($device_file);
-	my ($error, $grub_device) = $self->get_grub_device( $device_file);
-        return $grub_device if $error;
-
-        $conf_string =~ s/\$root/$device_file/g;
-        $conf_string =~ s/\$grubroot/(hd$grub_device,$partition_number)/g;
-
-        return $self->write_menu_lst($conf_string, "truncate");
-};
-
-
-
-=head2 prepare_boot
-
-Make installed system ready for boot from hard disk.
-
-@return success - 0
-@return error   - error string
-
-=cut
-
-method prepare_boot()
-{
-        my $retval = 0;
-        return $retval if $retval = $self->configure_fstab();
-        return $retval if $retval = $self->generate_grub_menu_lst( );
-	return $retval if $retval = $self->generate_pxe_grub();
-# 	return $retval if $retval = $self->copy_menu_lst();
-        return 0;
-};
-
-
-=head2 create_mount_point
-
-Create the mount point if its not an existing directory.
-
-@param string - mount point path
-
-@return success - 0
-@return error   - error string
-
-
-=cut
-
-method create_mount_point($mount_point)
-{
-	# Creates mount_point if not exists
-        if (not -d $mount_point) {
-                unlink $mount_point;
-                system("mkdir","-p","$mount_point") and return ("Can't create mount point $mount_point");
-        }
-        return 0;
-};
-
 =head2 install_image
 
 Install an image on a given device and mount it to a given mount point. Make
@@ -471,8 +455,9 @@ sure to set partition label reasonably.
 
 =cut
 
-method install_image($image_file, $device_file, $mount_point)
+sub install_image
 {
+        my ($self, $image_file, $device_file, $mount_point) = @_;
  	my ($error, $partition_size)=$self->log_and_exec("/sbin/blockdev --getsize64 $device_file");
         if ($error) {
                 $self->log->warn("Can't get size of partition $device_file: $partition_size. Won't check if images fits.");
@@ -503,7 +488,7 @@ method install_image($image_file, $device_file, $mount_point)
                 return("Imagetype could not be detected");
 	}
         return 0;
-};
+}
 
 
 =head2 install_image_iso
@@ -521,9 +506,9 @@ Install an image of type iso.
 
 =cut
 
-method install_image_iso($image_file, $partition_size, $device_file, $mount_point)
-
+sub install_image_iso
 {
+        my ($self, $image_file, $partition_size, $device_file, $mount_point) = @_;
         # -s return the size in byte
         if ( $partition_size and (-s $image_file) > $partition_size) {
                 return("Image $image_file is to big for device $device_file");
@@ -532,10 +517,9 @@ method install_image_iso($image_file, $partition_size, $device_file, $mount_poin
         my $retval;
         return $retval if $retval=$self->log_and_exec("dd if=$image_file of=$device_file");
         return $retval if $retval=$self->log_and_exec("mount $device_file $mount_point");
-        $self->images([ @{$self->images}, $mount_point ]);
 
         return(0);
-};
+}
 
 
 =head2 install_image_tar
@@ -554,8 +538,9 @@ Install an image of type tar.
 
 =cut
 
-method install_image_tar($image_file, $partition_size, $device_file, $mount_point, $partition_label)
+sub install_image_tar
 {
+        my ($self, $image_file, $partition_size, $device_file, $mount_point, $partition_label) = @_;
         # -s return the size in byte
         if ( $partition_size and (-s $image_file) > $partition_size) {
                 return("Image $image_file is to big for device $device_file");
@@ -564,11 +549,10 @@ method install_image_tar($image_file, $partition_size, $device_file, $mount_poin
         my $retval;
         return $retval if $retval=$self->log_and_exec("mkfs.ext3 -q -L $partition_label $device_file");
         return $retval if $retval=$self->log_and_exec("mount $device_file $mount_point");
-        $self->images([ @{$self->images}, $mount_point ]);
 
         return $retval if $retval=$self->log_and_exec("tar xf $image_file -C $mount_point");
         return 0;
-};
+}
 
 =head2 install_image_gz
 
@@ -585,8 +569,9 @@ Install an image of type tar.gz.
 
 =cut
 
-method install_image_gz($image_file, $partition_size, $device_file, $mount_point, $partition_label)
+sub install_image_gz
 {
+        my ($self, $image_file, $partition_size, $device_file, $mount_point, $partition_label) = @_;
         my $gz_factor=3.82;
         if ( $partition_size and (-s $image_file)*$gz_factor > $partition_size) {
                 return("Image $image_file is to big for device $device_file");
@@ -595,11 +580,10 @@ method install_image_gz($image_file, $partition_size, $device_file, $mount_point
         my $retval;
         return $retval if $retval=$self->log_and_exec("mkfs.ext3 -q -L $partition_label $device_file");
         return $retval if $retval=$self->log_and_exec("mount $device_file $mount_point");
-        $self->images([ @{$self->images}, $mount_point ]);
 
         return $retval if $retval=$self->log_and_exec("tar xfz $image_file -C $mount_point");
         return 0;
-};
+}
 
 =head2 install_image_bz2
 
@@ -616,8 +600,9 @@ Install an image of type tar.bz2.
 
 =cut
 
-method install_image_bz2($image_file, $partition_size, $device_file, $mount_point, $partition_label)
+sub install_image_bz2
 {
+        my ($self, $image_file, $partition_size, $device_file, $mount_point, $partition_label) = @_;
         my $bz2_factor=4.30;
         if ( $partition_size and (-s $image_file)*$bz2_factor > $partition_size) {
                 return("Image $image_file is to big for device $device_file");
@@ -626,11 +611,10 @@ method install_image_bz2($image_file, $partition_size, $device_file, $mount_poin
         my $retval;
         return $retval if $retval=$self->log_and_exec("mkfs.ext3 -q -L $partition_label $device_file");
         return $retval if $retval=$self->log_and_exec("mount $device_file $mount_point");
-        $self->images([ @{$self->images}, $mount_point ]);
 
         return $retval if $retval=$self->log_and_exec("tar xfj $image_file -C $mount_point");
         return 0;
-};
+}
 
 
 =head2 copy_image
@@ -646,8 +630,9 @@ Install an image to a given partition and mount it to a given mount point.
 
 =cut
 
-method copy_image($device_file, $image_file, $mount_point)
+sub copy_image
 {
+        my ($self, $device_file, $image_file, $mount_point) = @_;
         $image_file = $self->cfg->{paths}{image_dir}.$image_file if not -e $image_file;
 
 	# Image exists?
@@ -655,7 +640,7 @@ method copy_image($device_file, $image_file, $mount_point)
                 return("Image $image_file could not be found");
         }
 	return $self->install_image($image_file, $device_file, $mount_point);
-};
+}
 
 =head2 unmount
 
@@ -670,8 +655,11 @@ order.
 sub unmount
 {
         my ($self) = @_;
+        my $basedir = $self->cfg->{paths}{base_dir};
         foreach my $image (reverse @{$self->images}) {
-                $self->log_and_exec('umount', $image);
+                my $mount = $basedir.$image->{mount};
+                next unless -d $mount;
+                $self->log_and_exec('umount', $mount);
         }
         return 0;
 }
